/**
 * JASMINe
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JAggregateCollector.java 8666 2011-07-26 09:33:03Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.probe.collectors.aggregate.internal;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.ow2.jasmine.probe.JasmineIndicValue;
import org.ow2.jasmine.probe.JasmineIndicator;
import org.ow2.jasmine.probe.JasmineSingleResult;
import org.ow2.jasmine.probe.collector.JasmineAggregateService;
import org.ow2.jasmine.probe.collector.JasmineCollector;
import org.ow2.jasmine.probe.collector.JasmineCollectorException;
import org.ow2.jasmine.probe.collectors.JCollector;
import org.ow2.jasmine.probe.probemanager.ProbeManager;

/**
 * Aggregate collector implementation.
 * @author danesa
 */
public class JAggregateCollector extends JCollector {

    /**
     * Built-in aggregate operation code.
     */
    private int ope;

    /**
     * Requested operation (aggregate function).
     */
    private String operation;

    /**
     * List of supported operations
     */
    public final static int OP_MIN = 1;
    public final static int OP_MAX = 2;
    public final static int OP_SUM = 3;
    public final static int OP_AVERAGE = 4;

    private JasmineCollector source = null;

    /**
     * Name of the field to merge
     */
    private String merged;

    /**
     * Need ProbeManager provided by the AggregateCollectorService.
     */
    private ProbeManager probeManager = null;

    /**
     * AggregateCollectorService sets ProbeManager reference.
     * @param probeManager
     */
    protected void setProbeManager(ProbeManager probeManager) {
        this.probeManager = probeManager;
    }
    /**
     * Constructor
     * @param name
     * @param def
     * @param period
     * @param source
     */
    public JAggregateCollector(String name, JasmineIndicator def, int period, String operation, JasmineCollector source, String merged) {
        super(name, def, period);
        this.operation = operation;
        ope = 0;
        if (operation.equalsIgnoreCase("min")) {
            ope = OP_MIN;
        } else if (operation.equalsIgnoreCase("max")) {
            ope = OP_MAX;
        }  else if (operation.equalsIgnoreCase("sum")) {
            ope = OP_SUM;
        }  else if (operation.equalsIgnoreCase("average")) {
            ope = OP_AVERAGE;
        }
        this.ope = ope;
        this.source = source;
        this.merged = merged;
    }

    /**
     * return the last result for this indicator
     * @return JasmineIndicValue
     * @throws JasmineCollectorException
     */
    @Override
    public JasmineIndicValue getLastResult() throws JasmineCollectorException {
        logger.debug("");
        JasmineIndicValue current = source.getLastResult();
        if (current == null) {
            return null;
        }

        JasmineAggregateService aggserv = probeManager.getAggregate(operation);
        if (aggserv != null) {
            // Got a service to do aggregation
            return aggserv.aggregate(current);
        }

        /*
         * Otherwise, get back to old code for the moment...
         */
        if (ope == OP_MAX) {
            return max(current);
        }

        throw new JasmineCollectorException("Unimplemented " + operation + " aggregate operation");
    }

    /**
     * Stop polling
     */
    public void stopPolling() {
        logger.debug("");
        // nothing to do
    }


    // ---------------------------------------------------------------------------
    // private methods
    // ---------------------------------------------------------------------------

    /**
     * Implements the "max" operation.
     * @param map Map of values to aggregate
     * @return value
     * @throws JasmineCollectorException
     */
    private Number max(Map<String, Number> map) throws JasmineCollectorException {
        Number ret = null;
        for (Number val : map.values()) {
            if (ret == null) {
                ret = val;
            } else {
                if (diffValues(val, ret) > 0) {
                    ret = val;
                }
            }
        }
        return ret;
    }

    /**
     * Implements the "min" operation.
     * @param map Map of values to aggregate
     * @return value
     * @throws JasmineCollectorException
     */
    private Number min(Map<String, Number> map) throws JasmineCollectorException {
        Number ret = null;
        for (Number val : map.values()) {
            if (ret == null) {
                ret = val;
            } else {
                if (diffValues(val, ret) < 0) {
                    ret = val;
                }
            }
        }
        return ret;
    }

    /**
     * Implements the "sum" operation.
     * @param map Map of values to aggregate
     * @return value
     * @throws JasmineCollectorException
     */
    private Number sum(Map<String, Number> map) throws JasmineCollectorException {
        Number ret = addValues(map.values());
        return ret;
    }

    /**
     * Implements the "average" operation.
     * @param map Map of values to aggregate
     * @return value
     * @throws JasmineCollectorException
     */
    private Number average(Map<String, Number> map) throws JasmineCollectorException {
        long ret = addValues(map.values());
        int nbval = map.values().size();
        return ret / nbval;
    }

    /**
     * Simple implementation for max operation.
     * Determine the maximum JasmineIndicValue by finding the maximum between all the JasmineIndicValue results
     * where the compared values are supposed to be non-null integers !!!
     * @param
     * @return
     * @throws JasmineCollectorException
     */
    private JasmineIndicValue max(JasmineIndicValue source) throws JasmineCollectorException {
        JasmineIndicValue maxIndicValue = new JasmineIndicValue();
        maxIndicValue.setName(source.getName());
        Collection<JasmineSingleResult> results = source.getValues();
        if (results.isEmpty()) {
            maxIndicValue.setValues(results);
            return maxIndicValue;
        }
        JasmineSingleResult maxResult = new JasmineSingleResult();
        Number maxValue = null;
        String maxName = null;
        HashMap<String, String> maxProps = null;
        for (JasmineSingleResult result : results) {
            // Get JasmineSingleResult with maximun value
            if (maxValue == null) {
                // initialize maxValue with the first value
                maxValue = result.getValue();
                maxName = result.getName();
                maxProps = result.getProperties();
            } else {
                Number value = result.getValue();
                if (diffValues(maxValue, value) < 0) {
                    maxValue = value;
                    maxName = result.getName();
                    maxProps = result.getProperties();
                }
            }
        }
        maxResult.setValue(maxValue);
        maxResult.setTimestamp(System.currentTimeMillis());
        maxResult.setName(maxName);
        maxResult.setProperties(maxProps);
        Collection<JasmineSingleResult> maxResults = new ArrayList<JasmineSingleResult>();
        maxResults.add(maxResult);
        maxIndicValue.setValues(maxResults);
        return maxIndicValue;
    }


}
